; qbwsock.asm
; Copyright 1999 by Richard Dawe
;
; This is distributed under the terms of the GNU Library General Public
; License. Please see the file lgpl.txt that came with this file.
;
; 16-bit code to access Winsock from a DOS box. This was written using the
; WSOCK.VXD Pseudo-Documentation - see:
;
; http://www.geocities.com/SiliconValley/Lab/3216/dl/wsockvxd.htm

[bits 16]
[uppercase]

[segment data]

wsock_name	db 'WSOCK.VXD'
wsock_entry	dd 0
wsock_id	dw 0x3e

%include "wsockvxd.inc"

vxdldr_entry	dd 0
vxdldr_id	dw 0x27

[segment code]

[global _qbwsock]

; --- qbwsock ---
;
; On entry from QuickBASIC, ds points to the program's data segment. All
; pointers are passed as near. QB vars as passed by reference, i.e. pointers.
;
; Args: 1st = return value, 2nd = command, 3rd = parameter structure
;
; Valid commands:
;	0	->	Initialise VxD (required)
;	1	->	Uninitialise VxD
;	0x100+	->	WSOCK.VXD functions, passed on

_qbwsock:
	push bp
	mov bp, sp

	; Execute the requested function
	mov di, [bp+8]	; Get argument 2, assuming near pointers
	mov ax, [di]

func0:
	; Initialise the VxDs
	cmp ax, 0
	jne func1
	call initvxd
	jmp done

func1:
	; Uninitialise the VxDs
	cmp ax, 1
	jne func_wsock
	call uninitvxd
	jmp done

func_wsock:
	; Check that this is a valid Winsock function
	; Store client DS in CX throughout, place it on stack too
	mov cx, ds
	push ds

	mov bx, data
	mov ds, bx
	cmp ax, [wsock_first_cmd]
	jl badfunc_wsock
	cmp ax, [wsock_last_cmd]
	jg badfunc_wsock

	; Execute a Winsock function
	; Address of parameter structure: ES:BX from client DS:[BP+10]
	mov es, cx
	mov bx, [bp+10]
	call far [wsock_entry]

	pop ds
	jmp done

badfunc_wsock:
	pop ds

badfunc:
	mov ax, 0xffff

done:
	; Store the return value
	mov di, [bp+6]
	mov word [di], ax

	mov sp, bp
	pop bp
	retf 2

; --- initvxd ---
;
; Load WSOCK.VXD & get its entry point

initvxd:
	push ds
	mov bx, data
	mov ds, bx

	; Get VXDLDR's entry point
	mov ax, 0x1684
	mov word bx, [vxdldr_id]
	int 0x2f

	; Check if it's valid
	mov bx, di
	cmp bx, 0
	jne vxdldr_ok
	mov bx, es
	cmp bx, 0
	jne vxdldr_ok
	mov ax, 0xffff
	jmp initvxd_done

vxdldr_ok:
	; Store VXDLDR's entry point
	mov bx, es
	mov word [vxdldr_entry],   di
	mov word [vxdldr_entry+2], bx

	; Load the VxD
	mov ax, 1
	mov bx, seg wsock_name
	mov ds, bx
	mov dx, wsock_name
	call far [vxdldr_entry]

	; Failed to load
	jnc wsock_loaded
	mov ax, 0xffff
	jmp initvxd_done

wsock_loaded:
	; Get WSOCK.VXD's entry point
	mov ax, 0x1684
	mov word bx, [wsock_id]
	int 0x2f

	; Check if it's valid
	mov bx, di
	cmp bx, 0
	jne wsock_ok
	mov bx, es
	cmp bx, 0
	jne wsock_ok
	mov ax, 0xffff
	jmp initvxd_done

wsock_ok:
	; Store VXDLDR's entry point
	mov bx, es
	mov word [wsock_entry],   di
	mov word [wsock_entry+2], bx

initvxd_ok:
	; It's completed successfully
	mov ax, 0

initvxd_done:
	pop ds
	retn

; --- uninitvxd ---
;
; Unload the VxD

uninitvxd:
	push ds
	mov bx, data
	mov ds, bx

	; Check we've got an entry point for VXDLDR
	mov word ax, [vxdldr_entry]
	cmp ax, 0
	jne wsock_unload
	mov word ax, [vxdldr_entry+2]
	cmp ax, 0
	jne wsock_unload
	mov ax, 0xffff
	jmp uninitvxd_done

wsock_unload:
	; Use VXDLDR to unload WSOCK.VXD
	mov bx, data
	mov ds, bx
	mov dx, wsock_name
	mov ax, 2
	call far [vxdldr_entry]
	jnc uninitvxd_ok
	mov ax, 0xffff
	jmp uninitvxd_done

uninitvxd_ok:
	mov ax, 0

uninitvxd_done:
	pop ds
	retn
